/* dbminit.c - Open the file for dbm operations. This looks like the
   original interface for dbm use. */

/*  GNU DBM  - DataBase Manager (database subroutines) by Philip A. Nelson
    Copyright (C) 1989  Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You may contact the author by:
       e-mail:  phil@wwu.edu
      us-mail:  Philip A. Nelson
                Computer Science Department
                Western Washington University
                Bellingham, WA 98226
        phone:  (206) 676-3035
       
*************************************************************************/



#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "gdbm.h"
#include "extern.h"

/* Initialize dbm system.  FILE is a pointer to the file name without the
   ".dbm" extension.  If the file has a size of zero bytes, a file initialization
   procedure is performed, setting up the initial structure in the file.
   Any error detected will cause a return value of -1.  No errors cause the
   return value of 0. */

int
dbminit (file)
     char *file;
{
  /* Call the actual routine, saving the pointer to the file information. */
  _dbm_file = gdbm_open (file, 0, DBM_WRITER, NULL);


  /* Initialize the "static" memory. */
  _dbm_memory = NULL;

  /* If there was an error, return error value. */
  if (_dbm_file == NULL)
    return -1;

  /* Turn recovery procedures off.  (default for compability mode) */
  dbm_recovery (_dbm_file, 0);
  return 0;
}


/* dbmclose: Although not included in dbm, for gdbm we have an extra
   procedure that should be called before leaving.  This makes sure
   that the file is unlocked and ready to use again.  */

dbmclose()
{
  gdbm_close (_dbm_file);
}
