/* gdbmdelete.c - Remove the key and its associated data from the database. */

/*  GNU DBM  - DataBase Manager (database subroutines) by Philip A. Nelson
    Copyright (C) 1989  Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You may contact the author by:
       e-mail:  phil@wwu.edu
      us-mail:  Philip A. Nelson
                Computer Science Department
                Western Washington University
                Bellingham, WA 98226
        phone:  (206) 676-3035
       
*************************************************************************/



#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "gdbm.h"
#include "gdbmerrno.h"
extern gdbm_error gdbm_errno;

/* Remove the KEYed item and the KEY from the database DBF.  The file on disk is
   updated to reflect the structure of the new database before returning from
   this procedure.  */

int
gdbm_delete (dbf, key)
     dbm_file_info *dbf;
     datum key;
{
  int elem_loc;		/* The location in the current hash bucket. */
  int last_loc;		/* Last location emptied by the delete.  */
  int home;		/* Home position of an item. */
  bucket_element elem;  /* The element to be deleted. */
  int num_bytes;	/* Used in file operations. */

  /* First check to make sure this guy is a writer. */
  if (dbf->read_write != DBM_WRITER)
    {
      gdbm_errno = READER_CANT_DELETE;
      return -1;
    }

  /* Find the item. */
  elem_loc = _dbm_findkey (dbf, key);
  if (elem_loc == -1)
    {
      gdbm_errno = ITEM_NOT_FOUND;
      return -1;
    }

  /* Save the element.  */
  elem = dbf->bucket->h_table[elem_loc];

  /* Delete the element.  */
  dbf->bucket->h_table[elem_loc].hash_value = -1;
  dbf->bucket->count -= 1;

  /* Move other elements to guarantee that they can be found. */
  last_loc = elem_loc;
  elem_loc = (elem_loc + 1) % dbf->header.bucket_elems;
  while (elem_loc != last_loc && dbf->bucket->h_table[elem_loc].hash_value != -1)
    {
      home = dbf->bucket->h_table[elem_loc].hash_value % dbf->header.bucket_elems;
      if ( (last_loc < elem_loc && (home <= last_loc || home > elem_loc))
	  || (last_loc > elem_loc && home <= last_loc && home > elem_loc))
	
	{
	  dbf->bucket->h_table[last_loc] = dbf->bucket->h_table[elem_loc];
	  dbf->bucket->h_table[elem_loc].hash_value = -1;
	  last_loc = elem_loc;
	}
      elem_loc = (elem_loc + 1) % dbf->header.bucket_elems;
    }

  /* Update state of current bucket. */
  _dbm_update (dbf, 0x77777709, elem.data_pointer, elem.key_size+elem.data_size,
		 dbf->bucket_adr, dbf->bucket->count);
  num_bytes = lseek (dbf->desc, dbf->bucket_adr, L_SET);
  if (num_bytes != dbf->bucket_adr) _dbm_fatal (dbf, "lseek error");
  num_bytes = write (dbf->desc, dbf->bucket, dbf->header.bucket_size);
  if (num_bytes != dbf->header.bucket_size) _dbm_fatal (dbf, "write error");
  fsync (dbf->desc);

  /* Free the file space. */
  _dbm_free (dbf, elem.data_pointer, elem.key_size+elem.data_size);

  /* Everything is ok. */
  _dbm_end_update (dbf);
  return 0;  
}
