/* gdbmfetch.c - Find a key and return the associated data.  */

/*  GNU DBM  - DataBase Manager (database subroutines) by Philip A. Nelson
    Copyright (C) 1989  Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You may contact the author by:
       e-mail:  phil@wwu.edu
      us-mail:  Philip A. Nelson
                Computer Science Department
                Western Washington University
                Bellingham, WA 98226
        phone:  (206) 676-3035
       
*************************************************************************/


#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "gdbm.h"
#include "gdbmerrno.h"
extern gdbm_error gdbm_errno;

/* Look up a given KEY and return the information associated with that KEY.
   The pointer in the structure that is  returned is a pointer to dynamically
   allocated memory block.  */

datum
gdbm_fetch (dbf, key)
     dbm_file_info *dbf;
     datum key;
{
  datum  return_val;		/* The return value. */
  int    elem_loc;		/* The location in the bucket. */
  int    num_bytes;		/* Number of bytes read. */

  /* Set the default return value. */
  return_val.dptr = NULL;

  /* Find the key. */
  elem_loc = _dbm_findkey (dbf, key);

  /* Read the data if the key was found.  */
  if (elem_loc >= 0)
    {
      /* This is the item.  Read and return the associated data. */
      return_val.dsize = dbf->bucket->h_table[elem_loc].data_size;
      return_val.dptr = (char *) malloc (return_val.dsize);
      if (return_val.dptr == NULL) _dbm_fatal (dbf, "malloc failed");
      num_bytes = read (dbf->desc, return_val.dptr, return_val.dsize);
      if (num_bytes != return_val.dsize) _dbm_fatal (dbf, "read error");
    }

  /* Check for an error and return. */
  if (return_val.dptr == NULL) gdbm_errno = ITEM_NOT_FOUND;
  return return_val;
}


