/* seq.c - Routines to visit all keys.  Not in sorted order. */

/*  GNU DBM  - DataBase Manager (database subroutines) by Philip A. Nelson
    Copyright (C) 1989  Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    You may contact the author by:
       e-mail:  phil@wwu.edu
      us-mail:  Philip A. Nelson
                Computer Science Department
                Western Washington University
                Bellingham, WA 98226
        phone:  (206) 676-3035
       
*************************************************************************/


#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "gdbm.h"
#include "extern.h"

/* Start the visit of all keys in the database.  This produces something in
   hash order, not in any sorted order.  */

datum
firstkey ()
{
  datum ret_val;

  /* Free previous dynamic memory, do actual call, and save pointer to new memory. */
  ret_val = gdbm_firstkey (_dbm_file);
  if (_dbm_memory != NULL) free (_dbm_memory);
  _dbm_memory = ret_val.dptr;

  /* Return the new value. */
  return ret_val;
}


/* Continue visiting all keys.  The next key following KEY is returned. */

datum
nextkey (key)
     datum key;
{
  datum ret_val;

  /* Free previous dynamic memory, do actual call, and save pointer to new memory. */
  ret_val = gdbm_nextkey (_dbm_file, key);
  if (_dbm_memory != NULL) free (_dbm_memory);
  _dbm_memory = ret_val.dptr;

  /* Return the new value. */
  return ret_val;
}
